﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopBusinessLogic.BusinessLogics
{
    public class ShopLogic : IShopLogic
    {
        private readonly ILogger _logger;
        private readonly IShopStorage _shopStorage;
        private readonly IRepairStorage _repairStorage;

        public ShopLogic(ILogger<ShopLogic> Logger, IShopStorage ShopStorage, IRepairStorage RepairStorage)
        {
            _logger = Logger;
            _shopStorage = ShopStorage;
            _repairStorage = RepairStorage;
        }

        public List<ShopViewModel>? ReadList(ShopSearchModel? Model)
        {
            _logger.LogInformation("ReadList. ShopName:{ShopName}.Id:{ Id}", Model?.ShopName, Model?.Id);
            
            var List = Model == null ? _shopStorage.GetFullList() : _shopStorage.GetFilteredList(Model);
            
            if (List == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            
            _logger.LogInformation("ReadList. Count:{Count}", List.Count);
            return List;
        }

        public ShopViewModel? ReadElement(ShopSearchModel Model)
        {
            if (Model == null)
                throw new ArgumentNullException(nameof(Model));

            _logger.LogInformation("ReadElement. ShopName:{ShopName}.Id:{ Id}", Model.ShopName, Model.Id);
            
            var Element = _shopStorage.GetElement(Model);
            if (Element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            
            _logger.LogInformation("ReadElement find. Id:{Id}", Element.Id);
            return Element;
        }

        public bool Create(ShopBindingModel Model)
        {
            CheckModel(Model);
            
            if (_shopStorage.Insert(Model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            
            return true;
        }

        public bool Update(ShopBindingModel Model)
        {
            CheckModel(Model);
            
            if (_shopStorage.Update(Model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            
            return true;
        }

        public bool Delete(ShopBindingModel Model)
        {
            CheckModel(Model, false);
            _logger.LogInformation("Delete. Id:{Id}", Model.Id);
            
            if (_shopStorage.Delete(Model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            
            return true;
        }

        public bool MakeSupply(SupplyBindingModel Model)
        {
            if (Model == null)
                throw new ArgumentNullException(nameof(Model));

            if (Model.Count <= 0)
                throw new ArgumentException("Количество ремонтов должно быть больше 0");

            var Shop = _shopStorage.GetElement(new ShopSearchModel
            {
                Id = Model.ShopId
            });
            
            if (Shop == null)
                throw new ArgumentException("Магазина не существует");

            if (Shop.ShopRepairs.ContainsKey(Model.RepairId))
            {
                var OldValue = Shop.ShopRepairs[Model.RepairId];
                OldValue.Item2 += Model.Count;
                Shop.ShopRepairs[Model.RepairId] = OldValue;
            }
            else
            {
                var Repair = _repairStorage.GetElement(new RepairSearchModel
                {
                    Id = Model.RepairId
                });
                
                if (Repair == null)
                    throw new ArgumentException($"Поставка: Товар с id:{Model.RepairId} не найденн");

                Shop.ShopRepairs.Add(Model.RepairId, (Repair, Model.Count));
            }
           
            return true;
        }

        private void CheckModel(ShopBindingModel Model, bool WithParams=true)
        {
            if (Model == null)
                throw new ArgumentNullException(nameof(Model));

            if (!WithParams)
                return;

            if (string.IsNullOrEmpty(Model.Address))
                throw new ArgumentException("Адрес магазина длжен быть заполнен", nameof(Model.Address));

            if (string.IsNullOrEmpty(Model.ShopName))
                throw new ArgumentException("Название магазина должно быть заполнено", nameof(Model.ShopName));

            _logger.LogInformation("Shop. ShopName: {ShopName}. Address: {Address}. OpeningDate: {OpeningDate}. Id:{Id}", 
                Model.ShopName, Model.Address, Model.OpeningDate, Model.Id);
            
            var Element = _shopStorage.GetElement(new ShopSearchModel
            {
                ShopName = Model.ShopName
            });
            if (Element != null && Element.Id != Model.Id)
            {
                throw new InvalidOperationException("Магазин с таким названием уже есть");
            }
        }
    }
}
